<?php
namespace InternetGalerie\Igshop2\Controller;


use GeorgRinger\NumberedPagination\NumberedPagination;
use InternetGalerie\Igshop2\Domain\Model\Shop;
use InternetGalerie\Igshop2\Domain\Repository\CategoryRepository;
use InternetGalerie\Igshop2\Domain\Repository\CurrencyRepository;
use InternetGalerie\Igshop2\Domain\Repository\OptionColorRepository;
use InternetGalerie\Igshop2\Domain\Repository\OptionSizeRepository;
use InternetGalerie\Igshop2\Domain\Repository\OptionWithPriceRepository;
use InternetGalerie\Igshop2\Domain\Repository\ProductClassRepository;
use InternetGalerie\Igshop2\Domain\Repository\ProductRepository;
use InternetGalerie\Igshop2\Domain\Repository\ShopRepository;
use InternetGalerie\Igshop2\Utility\CartUtility;
use Psr\Http\Message\ResponseInterface;
use TYPO3\CMS\Core\Http\PropagateResponseException;
use TYPO3\CMS\Core\MetaTag\MetaTagManagerRegistry;
use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\PathUtility;
use TYPO3\CMS\Extbase\Pagination\QueryResultPaginator;
use TYPO3\CMS\Core\Pagination\ArrayPaginator;
use TYPO3\CMS\Extbase\Http\ForwardResponse;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * ProductController
 */
class ProductController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    private $jsRootPath ='EXT:igshop2/Resources/Public/JavaScript/';

    protected Shop $shop;
    protected CartUtility $cartUtility;
    protected CurrencyRepository $currencyRepository;
    protected CategoryRepository $categoryRepository;
    protected ProductClassRepository $productClassRepository;
    protected ProductRepository $productRepository;
    protected OptionColorRepository $optionColorRepository;
    protected OptionSizeRepository $optionSizeRepository;
    protected OptionWithPriceRepository $optionWithPriceRepository;
    protected ShopRepository $shopRepository;

    public function __construct(
        CartUtility $cartUtility,
        CurrencyRepository $currencyRepository,
        CategoryRepository $categoryRepository,
        ProductClassRepository $productClassRepository,
        ProductRepository $productRepository,
        OptionColorRepository $optionColorRepository,
        OptionSizeRepository $optionSizeRepository,
        OptionWithPriceRepository $optionWithPriceRepository,
        ShopRepository $shopRepository
    ) {
        $this->cartUtility = $cartUtility;
        $this->currencyRepository = $currencyRepository;
        $this->categoryRepository = $categoryRepository;
        $this->productClassRepository = $productClassRepository;
        $this->productRepository = $productRepository;
        $this->optionColorRepository = $optionColorRepository;
        $this->optionSizeRepository = $optionSizeRepository;
        $this->optionWithPriceRepository = $optionWithPriceRepository;
        $this->shopRepository = $shopRepository;
    }

    public function initializeAction(): void
    {
        if($this->settings['shopUid']) {
            $this->shop = $this->shopRepository->findByUid($this->settings['shopUid']);
        }
    }

    /*
    protected function initializeAction() {
        //$this->productRepository->setSettings($this->settings);
        //add css for jquery ui

        // add js for images

        //$this->jsRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::siteRelPath($this->request->getControllerExtensionKey()) . 'Resources/Public/JavaScript/';

        if($this->settings['shopJS']){
            if(strpos($this->settings['shopJS'], 'EXT:igshop2/') !== false) {
                $this->settings['shopJS'] = str_replace('EXT:igshop2/', \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::siteRelPath('igshop2'), $this->settings['shopJS']);
            }

            $this->jsRootPath = $this->settings['shopJS'];
        }

        if($this->settings['compMode']){
            $this->includeJS($this->jsRootPath.'Products/igShop_Products_AjaxCart.js');
            $this->includeJS($this->jsRootPath.'Products/igShop_Products_PriceCalc.js');
        }

        $this->includeJS($this->jsRootPath.'Products/igShop_Products_imgSlider.js');
        // add js for amount input
        $this->includeJS($this->jsRootPath.'igShop_Products_amountInput.js');
        // add color animation for responsive buying of a product
        $this->includeJS($this->jsRootPath.'Lib/jquery-color/color.js');
        // Add jquery ui for money filter
        $this->includeJS($this->jsRootPath.'Lib/jquery-ui/jquery-ui.min.js');
        //Add Script from perfect Scrollbar - ohne IE8 Support
        $this->includeJS($this->jsRootPath.'Lib/perfect-scrollbar/perfect-scrollbar.jquery.min.js');
    }
    */
    public function includeJS($filename): void
    {
        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
        $pageRenderer->addJsFooterFile($filename);
    }

    public function listAction(): ResponseInterface
    {


        // get Request parameters
        $request = $this->request->getArguments();
        $search =(isset($request['search']) && is_array($request['search'])) ? $request['search'] : [];
        $search = array_merge(
            [
                'color' => $request['color'] ?? null,
                'size' => $request['size'] ?? null,
                'category' => $request['category'] ?? null,
                'productClass' => $request['productClass'] ?? null,
                'minPrice' => $request['minPrice'] ?? null,
                'maxPrice' => $request['maxPrice'] ?? null,
                'properties' => $request['properties'] ?? []
            ],
            $search
        );
        $filter = array_merge(
            [
                'colors' => $request['color'] ?? null,
                'sizes' => $request['size'] ?? null,
                'categories' => isset($request['category']) ? $request['category'] : ($request['categories'] ?? null),
                'productClasses' => $request['productClass'] ?? null,
                'minPrice' => $request['minPrice'] ?? null,
                'maxPrice' => $request['maxPrice'] ?? null,
                'properties' => $request['properties'] ?? []
            ],
            $search
        );


        $this->view->assign('search', $search);
        // Set sorting
        $sortSettings = '';
        if(strlen($this->settings['flexform']['userSorting']) > 0){
            $sortSettings = $this->settings['flexform']['userSorting'];
        }else{
            $sortSettings = $this->settings['productSorting'];
        }

        list($sorting, $sortWay) = explode('.', $sortSettings);

        //$request['sorting'] = 'weight';
        if(isset($request['sorting']) && $request['sorting']) {
            $sorting = $request['sorting'] ?? '';

            if(strpos($sorting, '.') !== false) {
                list($sorting, $sortWay) = explode('.', $sorting);
            }

            if(is_array($sorting)) {
                $sorting = implode(',', $sorting);
            }
        }
        //$request['sortWay'] = 'desc';
        if(isset($request['sortWay']) && $request['sortWay']) {
            $sortWay = $request['sortWay'] ?? null;
        }

        $this->view->assign('sorting', $sorting);
        $this->view->assign('sortWay', $sortWay);

        $possibleOptions = ['colors' => [], 'sizes' => [], 'minPrice' => null, 'maxPrice' => null];

        if(isset($request['searchString']) && $request['searchString']){
            $possibleProducts = $this->productRepository->findProducts($request['searchString']);
            foreach($possibleProducts AS $product){

                foreach($product->getOptionsColor() AS $color){
                    if(!in_array($color, $possibleOptions['colors'])){
                        $possibleOptions['colors'][] = $color;
                    }
                }

                foreach($product->getOptionsSize() AS $size){
                    if(!in_array($size, $possibleOptions['sizes'])){
                        $possibleOptions['sizes'][] = $size;
                    }
                }

                if($this->cartUtility->order->getCurrency()->getShort() == 'EUR'){
                    if($product->getEuroPrice() < $possibleOptions['minPrice'] || $possibleOptions['minPrice'] === null){
                        $possibleOptions['minPrice'] = $product->getPriceEuro();
                    }
                    if($product->getEuroPrice() > $possibleOptions['maxPrice'] || $possibleOptions['maxPrice'] === null){
                        $possibleOptions['maxPrice'] = $product->getPriceEuro();
                    }
                }else{
                    if($product->getPrice() < $possibleOptions['minPrice'] || $possibleOptions['minPrice'] === null){
                        $possibleOptions['minPrice'] = $product->getPrice();
                    }
                    if($product->getPrice() > $possibleOptions['maxPrice'] || $possibleOptions['maxPrice'] === null){
                        $possibleOptions['maxPrice'] = $product->getPrice();
                    }
                }
            }
            $this->view->assign('searchString', $request['searchString']);
        }


        // Get and Set Currency
        $currency = $this->cartUtility->order->getCurrency();

        if($this->currencyRepository->getSessionCurrency() == false){
            if( is_object($currency)) {
                $this->currencyRepository->setSessionCurrency($currency->getUid());
            }
        }

        // Filter Settings
        if($this->settings['flexform']['filterColor'] ?? false || $this->settings['flexform']['filterSize'] ?? false || $this->settings['flexform']['filterPrice'] ?? false || $this->settings['flexform']['filterCategory'] ?? false || $this->settings['flexform']['filterProductClass'] ?? false){
            $this->includeJS($this->jsRootPath.'Products/igShop_Products_filterForm.js');

            $this->view->assign('filterMode', true);

            // Settings / Request for Color
            if($this->settings['flexform']['filterColor'] ?? false){
                $this->view->assign('colors', [
                    'all' => ((isset($request['searchString']) && $request['searchString']) ? $possibleOptions['color'] : $this->optionColorRepository->findAll()),
                    'selected' =>  $request['color'] ?? null
                ]);
            }

            // Settings / Request for Size
            if($this->settings['flexform']['filterSize'] ?? false){
                $this->view->assign('sizes', [
                    'all' => ((isset($request['searchString']) && $request['searchString']) ? $possibleOptions['size'] : $this->optionSizeRepository->findAll()),
                    'selected' => $request['size'] ?? null
                ]);
            }

            // Settings / Request for Price
            if($this->settings['flexform']['filterPrice'] ?? false) {
                $flexGroups = $this->settings['flexform']['group'];
                $catNavUids = $request['category'] ?? '';
                $minPrice = ((isset($request['searchString']) && $request['searchString']) ? $possibleOptions['minPrice'] : $this->productRepository->getMinPrice($flexGroups, $catNavUids, $currency));
                $maxPrice = ((isset($request['searchString']) && $request['searchString']) ? $possibleOptions['maxPrice'] : $this->productRepository->getMaxPrice($this->cartUtility->order->getCurrency()));

                $this->view->assign('prices', [
                    'all' => [
                        'minPrice' => $minPrice,
                        'maxPrice' => $maxPrice
                    ],
                    'selected' => [
                        'minPrice' => ((isset($request['minPrice']) && $request['minPrice']) ? $request['minPrice'] : $minPrice),
                        'maxPrice' => ((isset($request['maxPrice']) && $request['maxPrice']) ? $request['maxPrice'] : $maxPrice)
                    ]
                ]
                );
            }

            if($this->settings['flexform']['filterCategory'] ?? false){
                $this->view->assign('categories', [
                    'all' => $this->categoryRepository->findAllWithProducts($search['mandant'] ?? null),
                    'selected' => $request['category']
                ]);
            }

            if($this->settings['flexform']['filterProductClass'] ?? false){
                $this->view->assign('productClasses', [
                    'all' => $this->productClassRepository->findAll(),
                    'selected' => $request['productClass']
                ]);
            }
        } else if($this->settings['flexform']['textCategoryFilter'] ?? false && !$this->settings['flexform']['categoryFilterMainCategories']){
            $this->view->assign('categories', [
                'all' => $this->categoryRepository->findAllWithProducts($search['mandant'] ?? null),
                'selected' => $request['category'] ?? null
            ]);
        } else if($this->settings['flexform']['textCategoryFilter'] ?? false && $this->settings['flexform']['categoryFilterMainCategories'] ?? false) {
            $mainCategoryUids = GeneralUtility::trimExplode(',', $this->settings['flexform']['categoryFilterMainCategories']);
            $categories = [];

            foreach($mainCategoryUids as $mainCategoryUid) {
                $mainCategory = $this->categoryRepository->findByUid($mainCategoryUid);
                $subcategories = $this->categoryRepository->findByParent($mainCategory);
                $categories[$mainCategoryUid] = [
                    'main' => $mainCategory,
                    'all' => $subcategories,
                    'selected' => $request['categories'][$mainCategoryUid] ?? null
                ];
            }
            $this->view->assign('categories', $categories);
        }

        $properties = [];
        if(isset($request['properties']) && !empty($request['properties'])) {
            foreach($request['properties'] as $property) {
                $properties[$property['fieldName']] = $property['value'];
            }

            $this->view->assign('properties', $properties);
        }

        $productsPerPage = $this->settings['flexform']['productsPerPage'] ?? 0;
        if ($productsPerPage < 1 || empty($productsPerPage)) {
            $productsPerPage = 96;
        }

        // Get Products - filtered by settings
        $products = null;
        if($this->settings['flexform']['showOnlyActionPrices'] ?? false){
            $products = $this->productRepository->getProductsWithActionPrice($sorting, $sortWay);
        } else {
            $flexGroups = $this->settings['flexform']['group'];
            //var_dump($flexGroups);
            $catNavUids = $request['category'] ?? '';
            $this->view->assign('category', $catNavUids);

            $statusses = array_filter(GeneralUtility::trimExplode(',', $this->settings['flexform']['status'] ?? ''));
			$disableCatSort = $this->settings['disableCatSort'];

            if($catNavUids) {
                $this->view->assign('categoryObj', $this->categoryRepository->findByUid($catNavUids));
            } else if($flexGroups) {
                $this->view->assign('categoryObj', $this->categoryRepository->findByUid($flexGroups));
            }
            $products = $this->productRepository->findAll($flexGroups, $catNavUids, $sorting, $sortWay, $filter, $currency, $request['searchString'] ?? '', 0, $statusses,$disableCatSort, $properties);
        }

        if($this->settings['flexform']['productsPerPage'] > 0) {
            $currentPage = $this->request->hasArgument('currentPage') ? (int)$this->request->getArgument('currentPage') : 1;

            $paginator = null;

            if(is_array($products)) {
                $paginator = new ArrayPaginator($products, $currentPage, $this->settings['flexform']['productsPerPage']);
            } else {
                $paginator = new QueryResultPaginator($products, $currentPage, $this->settings['flexform']['productsPerPage']);
            }
            $pagination  = new NumberedPagination($paginator,10); // zweiter Argument: maximal Anzahl Links

            $this->view->assign('pagination', [
                'paginator' => $paginator,
                'pagination' => $pagination,
            ]);

            $this->view->assign('request', $request);
        }
        $this->view->assign('products', $products);

        // Set the Currency
        $this->view->assign('currencies', $this->shop->getCurrenciesAssociative());
        $this->view->assign('currency', $this->currencyRepository->getCurrency());

        // Assign Settings - abwärtskompatibilität

        $productsInRows = $this->settings['flexform']['elementsInRow'];
        if($productsInRows < 1 || empty($productsInRows))
            $productsInRows = 25;
        $this->view->assign('productsInRows', $productsInRows);

        $this->view->assign('productsPerPage', $productsPerPage);
        $this->view->assign('singlePage', $this->settings['flexform']['singlePage']);

        // Andere Page ID fuer Details
        $detailPageId = $this->settings['flexform']['detailPage'] ?? 0;
        if($detailPageId > 0){
            $this->view->assign('detailPage', $detailPageId);
        }
        $this->view->assign('detailPage',2122);

        $this->view->assign('productCount', count($products));
        $this->view->assign('cartPageId', $this->settings['cartPid']);
        $this->view->assign('request', $request);

        $this->includeJS($this->jsRootPath.'Products/igShop_Products.js');
        if( $this->settings['elevateZoom'] ) {
            $src = $this->jsRootPath.'Lib/Elevatezoom/jquery.elevateZoom.min.js';
            $this->includeJS($src, 'text/javascript', TRUE, FALSE, '', TRUE);
            $this->includeJS($this->jsRootPath.'Products/products-elevateZoom.js');
        }

        return $this->htmlResponse();
    }

    public function showTopProductAction(): ResponseInterface
    {
        $topProduct = null;
        if(isset($this->settings['flexform']['group']) && $this->settings['flexform']['group']) {
            $cateoryUids = GeneralUtility::trimExplode(',', $this->settings['flexform']['group']);
            $topProduct = $this->productRepository->findByCategories($cateoryUids);
        } else {
            $topProduct = $this->productRepository->findByUids($this->settings['flexform']['single']);
        }


        $this->view->assign('currencies', $this->shop->getCurrenciesAssociative());
        $this->view->assign('currency', $this->currencyRepository->getCurrency());

        $this->view->assign('products', $topProduct);

        /** View Assign Settings - abwärtskompatibilität **/

        if($this->settings['flexform']['singlePage'] > 0){
            $this->view->assign('singlePage', $this->settings['flexform']['singlePage']);
        }

        if($this->settings['flexform']['cropDescription'] > 0){
            $this->view->assign('cropDescription', $this->settings['flexform']['cropDescription']);
        }

        $this->view->assign('productsInRows', $this->settings['flexform']['elementsInRow']);
        $this->view->assign('productsPerPage', count($topProduct->toArray()));
        /*
        // Set Javascript for Show View
        $src = $this->jsRootPath.'Lib/Elevatezoom/jquery.elevateZoom.min.js';
        $this->includeJS($src, 'text/javascript', TRUE, FALSE, '', TRUE);
        */

        return $this->htmlResponse();
    }

    public function initializeShowAction(): void
    {
    }

    public function showAction(\InternetGalerie\Igshop2\Domain\Model\Product $product, bool $topProduct = false): ResponseInterface
    {
        $args = $this->request->getArguments();
        if (isset($this->settings['flexform']['single']) && $this->settings['flexform']['single'] > 0) {
            $productUid = intval($this->settings['flexform']['single']);
            $this->request->setArgument('product', $productUid);
        } else {
            $productUid = intval($args['product']);
        }
        $product = $this->productRepository->findByUid($productUid);
        $productNotAllowedOnPage = false;
        $flexGroups = $this->settings['flexform']['group'];
        if ($flexGroups && $product) {
            $categoryUids = GeneralUtility::intExplode(',', $flexGroups);

            if (!$product->hasCategoryIn($categoryUids)) {
                // Product is not allows according the groups/category settings
                $productNotAllowedOnPage = true;
            }
        }
        if ($productNotAllowedOnPage) {
            if ($categoryFirst = $product->getCategoryFirst()) {
                if ($detailPageUid = $categoryFirst->getDetailPageUid()) {
                    //var_dump($product->getCategoryFirst());die('detailPageUid='.  $detailPageUid .'(list='.$categoryFirst->getListPageUid().')');
                    //$this->request->setPluginName('FeshopShow');
                    return $this->redirect('show', 'Product', null, ['product' => $product], $detailPageUid, 0, 301); //301=Permanent Redirect
                }
                if ($listPageUid = $categoryFirst->getListPageUid()) {
                    //$this->request->setPluginName('Feshop');
                    return $this->redirect('show', 'Product', null, null, $listPageUid, 0, 301); //301=Permanent Redirect
                }
            }
            $product = null;
        }
        if (!$product) {
            if ($this->settings['flexform']['listPage'] > 0) {
                //$this->request->setPluginName('Feshop');
                return $this->redirect('list', 'Product', null, null, intval($this->settings['flexform']['listPage']));
            } else {
                return new ForwardResponse('list');
            }
        }
        
        //$product = $this->productRepository->findByUid($product->get_localizedUid());
        // Aktuelle Waehrung klaeren
        $defaultCountry = $this->shop->getDefaultCountry();
        $currencyDefault = $this->currencyRepository->findOneByUid($defaultCountry->getCurrency());
        if( ! $currencyDefault )
            return 'No default currency set';
        $currencyId = null;
        if( isset($_GET['currency']) ){
            //$_SESSION['currency'] = $_GET['currency'];
            $cr= $this->currencyRepository->findByShort( $_GET['currency'] );
            $c=$cr->getFirst();
            if($c)
                $currencyId = $c->getUid();
            else
                $currencyId=$currencyDefault->getUid();
        } else {
            $currencyId = isset($_REQUEST['tx_igshop2_feshop']) ? $_REQUEST['tx_igshop2_feshop']['currency']['uid'] : 0;
        }
        if ($currencyId > 0) {
            $this->currencyRepository->setSessionCurrency($currencyId);
        }

        $imageUrl = '';
        $metaTagManager = GeneralUtility::makeInstance(MetaTagManagerRegistry::class)->getManagerForProperty('og:title'); //OpenGraphMetaTagManager
        if(count($product->getImage()->toArray()) > 0){
            $imageUrl = 'http://'.$_SERVER['SERVER_NAME'].'/'.$product->getImage()->toArray()[0]->getOriginalResource()->getPublicUrl();
            $metaTagManager->addProperty('og:image', $imageUrl);
        }
        $metaTagManager->addProperty('og:url', $this->uriBuilder->getRequest()->getUri());
        $metaTagManager->addProperty('og:title', $product->getName());
        $metaTagManager->addProperty('og:description', $this->truncate(strip_tags($product->getDescription()), 250, '...'));

        // Nicht Hauptwährung, canonical Tag auf Default Produktseite
        if( ($currencyId!==null && $currencyId!=$currencyDefault->getUid()) ||  isset($_GET['currency'])) {
            $requestUri_exploded = explode('?',$this->uriBuilder->getRequest()->getUri());
            $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
            $pageRenderer->addHeaderData('<link ' .
                                                     GeneralUtility::implodeAttributes([
                                                         'rel' => 'canonical',
                                                         'href' =>$requestUri_exploded[0]
                                                     ]) .' />' . LF);
        }

        $GLOBALS['TSFE']->page['title'] = $product->getName();
        // set pagetitle for indexed search to news title
        $GLOBALS['TSFE']->indexedDocTitle = $product->getName();

        // Set Javascript for Show View
        $this->includeJS($this->jsRootPath.'Products/igShop_Products.js');
        if( $this->settings['elevateZoom'] ) {
            $src = $this->jsRootPath.'Lib/Elevatezoom/jquery.elevateZoom.min.js';
            $this->includeJS($src, 'text/javascript', TRUE, FALSE, '', TRUE);
            $this->includeJS($this->jsRootPath.'Products/products-elevateZoom.js');
        }

        if($this->settings['reminderList']['enable']) {
            $this->includeJS('EXT:igshop2/Resources/Public/JavaScript/ReminderList/reminderList.js');
        }


        // check if thumbs should be set
        $thumbs = 0;
        // check if separatet thumbs are set correctly (same amount)
        $separateThumbs = 0;

        if(count($product->getImage()) > 1 || (count($product->getVideos()) >= 1 && $product->getImage() >= 1)){
            $thumbs = 1;
            if(count($product->getImage()) == count($product->getImageThumb())){
                $separateThumbs = 1;
            }
        }

        $shareButtons = false;
        if(is_array($this->settings['flexform']['socialMedia'])){
            $shareButtons = true;
        }

        // Build the link to share
        $socialMediaLink = $this->uriBuilder
                         ->reset()
                         ->setTargetPageUid($GLOBALS['TSFE']->id)
                         ->setCreateAbsoluteUri(TRUE)
                         ->setArguments(['tx_igshop2_feshop' => ['product' => $product->getUid()]])
                         ->build();

        $this->view->assign('imageUrl', $imageUrl);
        $this->view->assign('socialMediaLink', urlencode($socialMediaLink));
        $this->view->assign('shareButtons', $shareButtons);

        $hasRelatedCategoryProducts = false;
        if($this->settings['flexform']['relatedProducts'] && $this->settings['flexform']['relatedProducts'] != 'onlyProdRelations' && count($product->getCategory()) > 0){
            // Get related products by category
            $relatedProducts = $this->productRepository->getByCategoryExcludeProduct($product);
            $this->view->assign('relatedProducts', $relatedProducts);
            //var_dump($relatedProducts); exit;
            if(count($relatedProducts) > 0)
                $hasRelatedCategoryProducts = true;
        }

        if($hasRelatedCategoryProducts || count($product->getProducts()) > 0)
            $this->view->assign('hasRelatedProducts', 1);

        $this->view->assign('product', $product);

        $this->view->assign('thumbs', $thumbs);
        $this->view->assign('separateThumbs', $separateThumbs);



        $this->view->assign('currency', $this->currencyRepository->getCurrency());
        $this->view->assign('currencies', $this->shop->getCurrenciesAssociative());


        /** Assign settings - abwärtskompatibilität **/

        if($this->settings['flexform']['listPage'] > 0){
            $this->view->assign('listPage', $this->settings['flexform']['listPage']);
        }

        $this->view->assign('thumbs', $thumbs);
        $this->view->assign('separateThumbs', $separateThumbs);
        $this->view->assign('ajaxProduct', $this->settings['ajaxProduct'] ?? 0);
        $this->view->assign('elevateZoom', $this->settings['elevateZoom'] ?? 0);
        $this->view->assign('cartPageId', $this->settings['cartPid'] ?? 0);
        $this->view->assign('prodLimit', $this->settings['prodLimit'] ?? 0);

        if($this->request->hasArgument('category')) {
            $categoryObj = $this->categoryRepository->findByUid($this->request->getArgument('category'));
            $this->view->assign('category', $this->request->getArgument('category'));
            $this->view->assign('categoryObj', $categoryObj);
        }

        return $this->htmlResponse();
    }

    public function exportAction(?string $currency = null): ResponseInterface
    { // () {
        //$excludeCategory = $this->categoryRepository->findByUid(11);
        //$products = $this->productRepository->findAllExceptCategory($excludeCategory);
        $products = $this->productRepository->findAllForExport();
        if( is_null($currency) ) {
            $currency=$_GET['currency'] ? $_GET['currency'] : 'CHF';
        }

        $exportProducts = [];
        $countTotal=0;
        $productsMissing=[];
        foreach( $products as $p ) {
            $hasValidOption = false;
            if($p->getOptionsWithPrice() && count($p->getOptionsWithPrice())>0 ) {
                //echo('OPTION: ' . $p->getName() .'('.$p->getGtin().','.$p->getMpn().')<br />');
                foreach( $p->getOptionsWithPrice() as $op ) {
                    if($op->getGtin() || $op->getMpn()) {
                        $hasValidOption = true;
                        $tp = clone($p);
                        $tp->setPrice($op->getPrice());
                        $tp->setPriceEuro($op->getPriceEuro());
                        if( $op->getGtin())
                            $tp->setGtin($op->getGtin());
                        if( $op->getMpn())
                            $tp->setMpn($op->getMpn());
                        if( $op->getName()) {
                            if(stristr($op->getName(), $tp->getName()) ) {
                                $tp->setName($op->getName()) ;
                            } else {
                                $tp->setName($tp->getName() . ' - ' . $op->getName()) ;
                            }
                        }
                        if( count($op->getImage()))
                            $tp->setImage($op->getImage());

                        $countTotal++;
                        if(( $tp->getGtin() ||$tp->getMpn()) && ($currency=='CHF' || $tp->getPriceEuro())) {
                            $exportProducts[]=[
                                'product' => $tp,
                                'option' => [
                                    'isOption' => true,
                                    'addId' => $op->getUid()
                                ]
                            ];
                        } else {
                            $productsMissing[]= $tp->getName() . '('.$op->getUid().')';
                        }
                    }
                }
            }
            if(!$p->getOptionsWithPrice() || !$hasValidOption) {
                $countTotal++;
                if(( $p->getGtin() ||$p->getMpn()) && ($currency=='CHF' || $p->getPriceEuro())) {
                    //echo('PROD: ' . $p->getName() .'('.$p->getGtin().','.$p->getMpn().')<br />');
                    $exportProducts[]=[
                        'product' => $p,
                        'option' => [
                            'isOption' => false,
                            //							 'addId' => null
                        ]
                    ];
                } else {
                    $productsMissing[]= $p->getName() . '('.$p->getUid().')';
                }
            }
        }
        if( isset($_GET['debug']) ){
            echo('Total Produkte und Optionen: '.$countTotal."<br />\n");
            echo('Exportierte Produkte und Optionen: '.count($exportProducts)."<br />\n");
            echo("<br /><b>Fehlende Produkte:</b><br />\n");
            echo(implode("<br />\n",$productsMissing));
            exit(0);
        }

        $this->view->assign('products', $exportProducts);
        $this->view->assign('currency', $currency);

        // wird wohl nicht mehr verwendet
        $site = $GLOBALS['TYPO3_REQUEST']->getAttribute('site');
        $base = $site->getAttribute('base');
        //$this->view->assign('baseUrl', 'http://' . $_SERVER['HTTP_HOST']);
        $this->view->assign('baseUrl', $base);

        $now = gmdate('D, d M Y H:i:s');
        $xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" . trim($this->view->render());
        $response = $this->responseFactory
            ->createResponse()
            ->withHeader('Content-Type', 'text/xml')
            ->withHeader('Expires', $now.' GMT')
            ->withHeader('Cache-Control', 'max-age=0, no-cache, must-revalidate, proxy-revalidate')
            ->withHeader('Last-Modified', $now.' GMT')
            ->withBody($this->streamFactory->createStream($xml));

        throw new PropagateResponseException($response, 200);
        /*
        $this->xml_send_headers();
        echo($xml);
        exit(0);

        return $this->htmlResponse();
        */
    }
    /*
    private function xml_send_headers(): void
    {
        // disable caching
        $now = gmdate('D, d M Y H:i:s');
        header('Expires: '.$now.' GMT');
        header('Cache-Control: max-age=0, no-cache, must-revalidate, proxy-revalidate');
        header('Last-Modified: '.$now.' GMT');

        // force download
        header('Content-Type: text/xml');
    }
    */
    public function searchResultsAction(): ResponseInterface
    {

        $arguments = $this->request->getArguments();

        $products = $this->productRepository->findProducts($arguments['searchString']);
        $this->view->assign('products', $products);

        $this->view->assign('currencies', $this->shop->getCurrenciesAssociative());
        $this->view->assign('currency', $this->currencyRepository->getCurrency());

        return $this->htmlResponse();
    }

    public function searchBoxAction(string $name = ''): ResponseInterface
    {
        $arguments = $this->request->getArguments();
        $searchString = $arguments['name'];

        $this->view->assign('searchString', $searchString);

        return $this->htmlResponse();
    }


    private function truncate(string $string, int $length=100, string $append='&hellip;'): string
    {
        $string = trim($string);

        if(strlen($string) > $length) {
            $string = wordwrap($string, $length);
            $string = explode("\n", $string, 2);
            $string = $string[0] . $append;
        }

        return $string;
    }

    public function ajaxGetProductPriceAction(): ResponseInterface
    {
        $arguments = $this->request->getArguments();

        $product = $this->productRepository->findbyUid($arguments['productid']);
        $price = $product->getFinalPrice($this->cartUtility->order->getCurrency());
        echo $this->renderFluid('Currency/PriceController.html', ['price' => $price*$arguments['amount'], 'currency' => $this->currencyRepository->getCurrency()]);
        exit;

        return $this->htmlResponse();
    }

    public function ajaxSearchAction(string $searchword): ResponseInterface
    {
        $currency = $this->cartUtility->order->getCurrency();
        $products = $this->productRepository->findAll(
            null,
            null,
            'sorting',
            'asc',
            [],
            null,
            $searchword,
            4
        );

        $categories = $this->categoryRepository->findBySearch($searchword, 4);

        if(\TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded('indexed_search_autocomplete')) {
            $searchService = GeneralUtility::makeInstance(\ID\IndexedSearchAutocomplete\Service\SearchService::class);
            $result = $searchService->searchAWord(['s' => $searchword], 10);
            $this->view->assign('autocompleteResults', $result['autocompleteResults']);
        }

        $this->view->assign('products', $products);
        $this->view->assign('categories', $categories);
        $this->view->assign('searchword', $searchword);

        echo $this->view->render();
        exit(0);

        return $this->htmlResponse();
    }

    private function renderFluid(string $template, array  $parameter = []): string
    {
        $fluidView = GeneralUtility::makeInstance(StandaloneView::class);
        $fluidView->getRequest()->setControllerExtensionName('igshop2');
        $conf = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName($conf['view']['templateRootPath']);
        $templateFile = $templateRootPath.$template;
        $partialOrder = $extPath . 'Resources/Private/Partials/';
        $fluidView->setTemplatePathAndFilename($templateFile);
        $fluidView->setPartialRootPath($conf['view']['partialRootPath']);
        if(!empty($parameter)) {
            $fluidView->assignMultiple($parameter);
        }
        return $fluidView->render();
    }

    public function htmlTitleAction(): ResponseInterface
    {
        $feShopGet = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP('tx_igshop2_feshop');

        if(is_array($feShopGet) && isset($feShopGet['product'])) {
            $productUid = $feShopGet['product'];
        }

        if($productUid) {
            $product=$this->productRepository->findByUid($productUid);
            $title = $product->getName();

            if($product->getCategoryTopLevel()) {
                $title .= ' - ' . $product->getCategoryTopLevel()->getName();
            }

            if($this->settings['titleSuffix']) {
                $title .= ' - ' . $this->settings['titleSuffix'];
            }

            return $this->htmlResponse('<title>' . $title . '</title>');
        }

        return $this->htmlResponse('');
    }

    public function sliderAction(): ResponseInterface
    {

        // Set sorting
        $sortSettings = '';
        if(strlen($this->settings['flexform']['userSorting']) > 0){
            $sortSettings = $this->settings['flexform']['userSorting'];
        }else{
            $sortSettings = $this->settings['productSorting'];
        }

        list($sorting, $sortWay) = explode('.', $sortSettings);

        $this->view->assign('sorting', $sorting);
        $this->view->assign('sortWay', $sortWay);

        // Get and Set Currency
        $currency = $this->cartUtility->order->getCurrency();

        if($this->currencyRepository->getSessionCurrency() == false){
            if( is_object($currency)) {
                $this->currencyRepository->setSessionCurrency($currency->getUid());
            }
        }
        $limit = intval($this->settings['flexform']['maxProducts']);
        if( $limit<0 ) {
            $limit=12;
        }
        // Get Products - filtered by settings
        $products = null;
        $flexGroups = null;
        if($this->settings['flexform']['showOnlyActionPrices']){
            $products = $this->productRepository->getProductsWithActionPrice($sorting, $sortWay);
        } else {
            $flexGroups = $this->settings['flexform']['group'];
            $statusses = array_filter(GeneralUtility::trimExplode(',', $this->settings['flexform']['status']));

            $this->view->assign('categoryObj', $this->categoryRepository->findByUid($flexGroups));
            // limit = 0, wegen page browser
            $products = $this->productRepository->findAll($flexGroups, null, $sorting, $sortWay, [], $currency, null, 0, $statusses);
        }

        $this->view->assign('products', $products);

        // Assign Settings - abwärtskompatibilität

        $this->view->assign('slider', [
            'slidesToShow' => $this->settings['flexform']['slidesToShow'] ? : 4,
            'speed' => $this->settings['flexform']['speed'] ? : 1000,
            'autoplaySpeed' => $this->settings['flexform']['autoplaySpeed'] ? : 1000,
            'settingsName' => $this->settings['flexform']['settingsName'] ? : 'homeSlider',
            'class' => $this->settings['flexform']['sliderClass'],
            'classname' => $this->settings['flexform']['sliderClassname'] ? : 'top-seller-product-slider',
        ]);


        // Andere Page ID fuer Details
        $detailPageId = $this->settings['flexform']['detailPage'];
        $this->view->assign('singlePage',  $this->settings['flexform']['detailPage']);
        if($detailPageId > 0){
            $this->view->assign('detailPage', $detailPageId);
        }
        //$this->view->assign('detailPage',2122);

        $this->view->assign('productCount', count($products));

        //$this->includeJS($this->jsRootPath.'Products/igShop_Products.js');
        return $this->htmlResponse();
    }



}
