<?php
declare(strict_types = 1);
namespace InternetGalerie\Igshop2\Form\Element;

use TYPO3\CMS\Backend\Form\Element\AbstractFormElement;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\StringUtility;

class PropertyFieldsElement extends AbstractFormElement
{
    public function render():array
    {
        $connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
        $qb = $connectionPool->getQueryBuilderForTable('tx_igshop2_domain_model_category');
        $row = $this->data['databaseRow'];

        $fieldInformationResult = $this->renderFieldInformation();
        $resultArray = $this->mergeChildReturnIntoExistingResult($this->initializeResultArray(), $fieldInformationResult, false);

        $categories = [];
        $properties = [];
        $propertyUids = [];

        // Only show properties when product was already saved
        if(is_numeric($row['uid']) && $row['uid'] > 0) {
            // Get categories of product
            $categories = $qb->select('*')
                            ->from('tx_igshop2_product_category_mm')
                            ->join(
                                'tx_igshop2_product_category_mm',
                                'tx_igshop2_domain_model_category',
                                'cat',
                                'cat.uid = tx_igshop2_product_category_mm.uid_foreign'
                            )
                            ->where('uid_local = ' . $row['uid'])
                            ->andWhere('deleted = 0 AND hidden = 0')
                            ->orderBy('tx_igshop2_product_category_mm.sorting')
                            ->executeQuery()
                            ->fetchAllAssociative();

            // Get all properties for the product categories
            foreach($categories as $category) {
                $qb = $connectionPool->getQueryBuilderForTable('tx_igshop2_domain_model_property');
                $propertiesForCategory = $qb->select('*')
                                            ->from('tx_igshop2_category_property_mm')
                                            ->join(
                                                'tx_igshop2_category_property_mm',
                                                'tx_igshop2_domain_model_property',
                                                'prop',
                                                'prop.uid = tx_igshop2_category_property_mm.uid_foreign'
                                            )
                                            ->where('uid_local = ' . $category['uid'])
                                            ->andWhere('deleted = 0 AND hidden = 0')
                                            ->orderBy('tx_igshop2_category_property_mm.sorting')
                                            ->executeQuery()
                                            ->fetchAllAssociative();

                foreach($propertiesForCategory as $property) {
                    if(!in_array($property['uid'], $propertyUids)) {
                        $properties[] = $property;
                        $propertyUids[] = $property['uid'];
                    }
                }
            }

            // If there are no categories on the product, show all properties
            if(empty($categories)) {
                $qb = $connectionPool->getQueryBuilderForTable('tx_igshop2_domain_model_property');
                $properties = $qb->select('*')
                                ->from('tx_igshop2_domain_model_property')
                                ->andWhere('deleted = 0 AND hidden = 0')
                                ->orderBy('sorting')
                                ->executeQuery()
                                ->fetchAllAssociative();
            }

            // Only do something if there are properties
            if(!empty($properties)) {
                $parameterArray = $this->data['parameterArray'];
                $productItemName = htmlspecialchars($parameterArray['itemFormElName']);
                $html = [];
                $i = 0;
                $sorting = 1;

                // Loop through properties and generate field html
                foreach($properties as $property) {
                    $qb = $connectionPool->getQueryBuilderForTable('tx_igshop2_domain_model_propertyvalue');
                    $existingPropertyvalue = $qb->select('*')
                                        ->from('tx_igshop2_domain_model_propertyvalue')
                                        ->where('product = ' . $row['uid'])
                                        ->andWhere('property = ' . $property['uid'])
                                        ->andWhere('deleted = 0 AND hidden = 0')
                                        ->executeQuery()
                                        ->fetchAssociative();

                    $prefix = htmlspecialchars('data[tx_igshop2_domain_model_propertyvalue][NEW' . $i . ']');
                    if($existingPropertyvalue) {
                        $prefix = htmlspecialchars('data[tx_igshop2_domain_model_propertyvalue][' . $existingPropertyvalue['uid'] . ']');
                    }
                    

                    $valueFieldName = $property['db_type'] ?? null;
                    $propertyPartial = $property['partial'];
                    
                    $itemValue = '';
                    if($existingPropertyvalue && $valueFieldName) {
                        $itemValue = (string)$existingPropertyvalue[$valueFieldName] ?? '';
                    }

                    $label = $property['name'];

                    if($propertyPartial == 'List') {
                        $label .= ' (Liste: Jede Zeile ist ein Listenpunkt)';
                    }

                    if($property['unit']) {
                        $label .= ' (' . $property['unit'] . ')';
                    }

                    $html[] = '<div class="form-group t3js-formengine-validation-marker t3js-formengine-palette-field">';
                    $html[] = '<label class="form-label t3js-formengine-label">' . $label . '</label>';
                    $html[] = '<div class="formengine-field-item t3js-formengine-field-item" style="padding: 5px;">';
                    $html[] =   '<div class="form-wizards-wrap">';
                    $html[] =      '<div class="form-wizards-element">';
                    $html[] =         '<div class="form-control-wrap">';
                    $html[] =            '<input type="hidden" value="' . $sorting++ . '" ';
                    $html[] =            'name="' . $prefix . '[sorting]" class="form-control"';
                    $html[] =            ' />';
                    $html[] =            '<input type="hidden" value="' . $row['sys_language_uid'] . '" ';
                    $html[] =            'name="' . $prefix . '[sys_language_uid]" class="form-control"';
                    $html[] =            ' />';
                    $html[] =            '<input type="hidden" value="' . $row['pid'] . '" ';
                    $html[] =            'name="' . $prefix . '[pid]" class="form-control"';
                    $html[] =            ' />';
                    $html[] =            '<input type="hidden" value="' . $row['uid'] . '" ';
                    $html[] =            'name="' . $prefix . '[product]" class="form-control"';
                    $html[] =            ' />';
                    $html[] =            '<input type="hidden" value="' . $property['uid'] . '" ';
                    $html[] =            'name="' . $prefix . '[property]" class="form-control"';
                    $html[] =            ' />';

                    switch($propertyPartial) {
                        case 'Checkbox':
                            $html[] = '<div class="form-check-propertyvalue">';
                            $html[] =   '<input id="radio-' . $property['uid'] . '-' . $valueFieldName .  '-1" type="radio" value="1" ';
                            $html[] =       'name="' . $prefix . '[' . $valueFieldName  . ']"' . ($itemValue ? ' checked="checked"' : '');
                            $html[] =   ' />';
                            $html[] =   '<label for="radio-' . $property['uid'] . '-' . $valueFieldName .  '-1">Ja</label>';
                            $html[] = '</div>';
                            $html[] = '<div class="form-check-propertyvalue">';
                            $html[] =   '<input id="radio-' . $property['uid'] . '-' . $valueFieldName .  '-0" type="radio" value="0" ';
                            $html[] =       'name="' . $prefix . '[' . $valueFieldName  . ']"' . (!$itemValue ? ' checked="checked"' : '');
                            $html[] =   ' />';
                            $html[] =   '<label for="radio-' . $property['uid'] . '-' . $valueFieldName .  '-0">Nein</label>';
                            $html[] = '</div>';
                            break;
                        default:
                            if($valueFieldName == 'property_value_textarea') {
                                $html[] = '<textarea name="' . $prefix . '[' . $valueFieldName  . ']" class="form-control">' . htmlspecialchars($itemValue, ENT_QUOTES) . '</textarea>';
                            } else {
                                $html[] = '<input type="text" value="' . htmlspecialchars($itemValue, ENT_QUOTES) . '" ';
                                $html[] =   'name="' . $prefix . '[' . $valueFieldName  . ']" class="form-control"';
                                $html[] = ' />';
                            }
                            break;
                    }
                    
                    $html[] =         '</div>';
                    $html[] =      '</div>';
                    $html[] =   '</div>';
                    $html[] = '</div>';
                    $html[] = '</div>';

                    $i++;
                }

                $html[] = '<input type="hidden" value="' . count($properties) . '" ';
                $html[] = 'name="' . $productItemName . '" class="form-control"';
                $html[] = ' />';
                $resultArray['html'] = implode(LF, $html);
            }

        }

        return $resultArray;
    }
}